# Copyright 2023 (c) Anna Schumaker.
"""Tests the `xfstestsdb delete` command."""
import errno
import io
import unittest
import unittest.mock
import xfstestsdb.delete
import tests.xunit


class TestDelete(unittest.TestCase):
    """Tests the `xfstestsdb delete` command."""

    def setUp(self):
        """Set up common variables."""
        self.xfstestsdb = xfstestsdb.Command()
        self.delete = self.xfstestsdb.commands["delete"]

    def test_init(self):
        """Check that the delete command was set up properly."""
        self.assertIsInstance(self.delete, xfstestsdb.commands.Command)
        self.assertIsInstance(self.delete, xfstestsdb.delete.Command)
        self.assertEqual(self.xfstestsdb.subparser.choices["delete"],
                         self.delete.parser)

    @unittest.mock.patch("sys.stdout", new_callable=io.StringIO)
    def test_delete(self, mock_stdout: io.StringIO):
        """Test the `xfstestsdb delete` command with valid input."""
        self.xfstestsdb.run(["new", "/dev/test"])
        self.xfstestsdb.run(["delete", "1"])
        self.assertRegex(mock_stdout.getvalue(), "run #1 has been deleted")

        cur = self.xfstestsdb.sql("SELECT COUNT(*) FROM xfstests_runs")
        self.assertEqual(cur.fetchone()["COUNT(*)"], 0)

    @unittest.mock.patch("sys.stdout", new_callable=io.StringIO)
    def test_delete_tags(self, mock_stdout: io.StringIO):
        """Test the `xfstestsdb delete` command on a tagged run."""
        self.xfstestsdb.run(["new", "/dev/test"])
        self.xfstestsdb.run(["tag", "1", "mytag"])
        self.xfstestsdb.run(["delete", "1"])

        cur = self.xfstestsdb.sql("SELECT COUNT(*) FROM tags")
        self.assertEqual(cur.fetchone()["COUNT(*)"], 0)

    @unittest.mock.patch("sys.stdout", new_callable=io.StringIO)
    def test_delete_xunit(self, mock_stdout: io.StringIO):
        """Test the `xfstestsdb delete` command on a run with xunit files."""
        self.xfstestsdb.run(["new", "/dev/test"])
        self.xfstestsdb.run(["xunit", "read", "1", str(tests.xunit.XUNIT_1)])
        self.xfstestsdb.run(["delete", "1"])

        cur = self.xfstestsdb.sql("SELECT COUNT(*) FROM xunits")
        self.assertEqual(cur.fetchone()["COUNT(*)"], 0)

    @unittest.mock.patch("sys.stderr", new_callable=io.StringIO)
    def test_delete_error(self, mock_stderr: io.StringIO):
        """Test the `xfstestsdb delete` command with invalid input."""
        with self.assertRaises(SystemExit):
            self.xfstestsdb.run(["delete"])
        self.assertRegex(mock_stderr.getvalue(),
                         "error: the following arguments are required: runid")

    @unittest.mock.patch("sys.stderr", new_callable=io.StringIO)
    def test_delete_enoent(self, mock_stderr: io.StringIO):
        """Test the `xfstestsdb delete` command with an invalid runid."""
        with self.assertRaises(SystemExit) as sys_exit:
            self.xfstestsdb.run(["delete", "2"])

        self.assertEqual(sys_exit.exception.code, errno.ENOENT)
        self.assertRegex(mock_stderr.getvalue(),
                         "error: run #2 does not exist")
