# Copyright 2023 (c) Anna Schumaker.
"""The `xfstestsdb xunit read` command."""
import argparse
import datetime
import dateutil.tz
import errno
import html
import pathlib
import re
import sys
import xml.etree.ElementTree
from .. import commands
from .. import sqlite


class Command(commands.Command):
    """The `xfstestsdb xunit read` command."""

    def __init__(self, subparser: argparse.Action,
                 sql: sqlite.Connection) -> None:
        """Set up the xunit read command."""
        super().__init__(subparser, sql, "read", help="read in an xunit file")
        self.parser.add_argument("--name", metavar="name", nargs="?", type=str,
                                 help="a name for the xunit file")
        self.parser.add_argument("runid", metavar="runid", nargs=1, type=int,
                                 help="runid of the xfstests run "
                                 "associated with the file")
        self.parser.add_argument("file", metavar="file", nargs=1,
                                 type=argparse.FileType('r'),
                                 help="path to the xunit file to be read")

    def sanitize_message(self, text: str, properties: dict) -> str | None:
        """Replace text with xunit property names to reduce duplication."""
        if text is not None:
            for key in ["TEST_DEV", "TEST_DIR", "SCRATCH_DEV", "SCRATCH_MNT"]:
                if (value := properties.get(key)):
                    text = re.sub(value, f"${key}", text)
            return text.strip()

    def elm_testcase(self, runid: int, xunit: str,
                     elm: xml.etree.ElementTree.Element,
                     properties: dict, namespace: str) -> tuple:
        """Extract testcase data for an xml element."""
        status = "passed"
        message = None
        stdout = None
        stderr = None

        for e in elm:
            if e.tag.startswith(namespace):
                e.tag = e.tag[len(namespace):]
            match e.tag:
                case "failure" | "skipped": status = e.tag
                case "system-out": stdout = html.unescape(e.text)
                case "system-err":
                    stderr = re.sub(r"([-+]{3}\s\S+)(\s.*\n)",
                                    lambda m: f"{m.group(1)}\n",
                                    html.unescape(e.text))
                    stderr = re.sub(r"[+]{3} /?(.*)/(\S+/\d+.*)\n",
                                    lambda m: f"+++ results/{m.group(2)}\n",
                                    stderr)
            message = elm[0].attrib["message"]

        return (runid, xunit, elm.attrib["name"], status, elm.attrib["time"],
                self.sanitize_message(message, properties),
                self.sanitize_message(stdout, properties),
                self.sanitize_message(stderr, properties))

    def do_command(self, args: argparse.Namespace) -> None:
        """Read in an xunit file."""
        if self.sql("SELECT rowid FROM xfstests_runs WHERE rowid=?",
                    args.runid[0]).fetchone() is None:
            print(f"error: run #{args.runid[0]} does not exist",
                  file=sys.stderr)
            sys.exit(errno.ENOENT)

        if (xunitname := args.name) is None:
            xunitname = pathlib.Path(args.file[0].name).stem

        root = xml.etree.ElementTree.parse(args.file[0]).getroot()
        match = re.match(r"{.*?}", root.tag)
        namespace = match.group(0) if match else ""

        properties = {e.attrib["name"]: e.attrib["value"] for e in root[0]}
        testcases = [self.elm_testcase(args.runid[0], xunitname,
                                       elm, properties, namespace)
                     for elm in root[1:]]

        timestamp = root.attrib.get("start_timestamp")
        if timestamp is None:
            timestamp = root.attrib["timestamp"]
        if timestamp[-3] != ":":
            timestamp = timestamp[:-2] + ":" + timestamp[-2:]
        timestamp = datetime.datetime.fromisoformat(timestamp)
        timestamp = timestamp.astimezone(dateutil.tz.tzutc())
        timestamp = timestamp.replace(tzinfo=None)

        cur = self.sql("""INSERT INTO xunits (runid, timestamp, name, hostname,
                                              tests, failed, skipped, time)
                       VALUES (?, ?, ?, ?, ?, ?, ?, ?)""",
                       args.runid[0], timestamp, xunitname,
                       root.attrib["hostname"], root.attrib["tests"],
                       root.attrib["failures"], root.attrib["skipped"],
                       root.attrib["time"])
        if cur is None:
            print(f"error: run #{args.runid[0]} already has "
                  f"xunit file '{xunitname}'")
            sys.exit(errno.EEXIST)

        self.sql.executemany("""INSERT INTO xunit_properties_view
                                           (runid, xunit, key, value)
                             VALUES (?, ?, ?, ?)""",
                             *[(args.runid[0], xunitname, key, value)
                               for (key, value) in properties.items()])

        self.sql.executemany("""INSERT INTO testcases_view
                                        (runid, xunit, testcase, status,
                                         time, message, stdout, stderr)
                             VALUES (?, ?, ?, ?, ?, ?, ?, ?)""", *testcases)

        print(f"added '{xunitname}' with {root.attrib['tests']} testcases "
              f"to run #{args.runid[0]}")
